@@ -17,7 +17,7 @@ _re_qm = re.compile(r"\?")
 _re_cir = re.compile(r"\^")
 
 T = TypeVar("T")
-ElementsType = Union[Collection[str], Collection[T], OrderedDictType[T, float]]
+ElementsType = Union[Collection[T], OrderedDictType[T, float]]
 
 
 class BaseProvider:
@@ -108,6 +108,7 @@ class BaseProvider:
             "NG",
             "NZ",
             "PH",
+            "PK",
             "SG",
             "US",
             "ZA",
@@ -532,7 +533,7 @@ class BaseProvider:
         """
         return self.random_elements(elements, length, unique=False)
 
-    def random_element(self, elements: ElementsType[T] = ("a", "b", "c")) -> T:
+    def random_element(self, elements: ElementsType[T] = ("a", "b", "c")) -> T:  # type: ignore[assignment]
         """Generate a randomly sampled object from ``elements``.
 
         For information on the ``elements`` argument, please refer to
@@ -645,6 +646,10 @@ class BaseProvider:
         """Generate a string with each placeholder in ``text`` replaced according to the following rules:
 
         - Number signs ('#') are replaced with a random digit (0 to 9).
+        - Percent signs ('%') are replaced with a random non-zero digit (1 to 9).
+        - Dollar signs ('$') are replaced with a random digit above two (2 to 9).
+        - Exclamation marks ('!') are replaced with a random digit or an empty string.
+        - At symbols ('@') are replaced with a random non-zero digit or an empty string.
         - Question marks ('?') are replaced with a random character from ``letters``.
 
         By default, ``letters`` contains all ASCII letters, uppercase and lowercase.
@@ -656,6 +661,7 @@ class BaseProvider:
         :sample: letters='ABCDE'
         :sample: text='Product Number: ????-########'
         :sample: text='Product Number: ????-########', letters='ABCDE'
+        :sample: text='Order: ##??-$'
         """
         return self.lexify(self.numerify(text), letters=letters)
 
