@@ -529,6 +529,8 @@ class Provider(AddressProvider):
         "Vizovická",
         "K Brusce",
         "Mírová",
+        "Písnická",
+        "Durychova",
         "Rašínská",
         "Boušova",
         "Pobřežní",
@@ -550,6 +552,14 @@ class Provider(AddressProvider):
         "Vlašimská",
         "Nad Rohatci",
         "Tylišovská",
+        "Liškova",
+        "Kunratická",
+        "Branická",
+        "Na Strži",
+        "Višňová",
+        "Sulická",
+        "Zálesí",
+        "Vídeňská",
         "Nábřeží Kapitána Jaroše",
         "Lešovská",
         "U Podjezdu",
@@ -652,6 +662,8 @@ class Provider(AddressProvider):
         "Čerpadlová",
         "Vítězná",
         "Nad Plynovodem",
+        "Novodvorská",
+        "Budějovická",
         "U Smíchovského Hřbitova",
         "Nedvědovo Náměstí",
         "Bachova",
@@ -925,6 +937,38 @@ class Provider(AddressProvider):
         "Poleradská",
         "Jilmová",
         "Hostýnská",
+        "Otradovická",
+        "Cihlářova",
+        "Opavská",
+        "Hradecká",
+        "Vinohradská",
+        "Pařížská",
+        "Evropská",
+        "Mírová",
+        "Mlýnská",
+        "Pražská",
+        "Teplická",
+        "Tovární",
+        "V Lipách",
+        "Svatoplukova",
+        "Purkyňova",
+        "Na Letné",
+        "Bořivojova",
+        "U Hřbitova",
+        "Akátova",
+        "Plynárenská",
+        "Komenského",
+        "Havlíčkova",
+        "Husova",
+        "Na Nivách",
+        "Jandova",
+        "Jugoslávská",
+        "Pavlova",
+        "Kosmonautů",
+        "Svornosti",
+        "Moravská",
+        "Souběžná",
+        "Hasičská",
     )
 
     states = (
